package javax.swing;

import javax.accessibility.AccessibleRelation;
import java.beans.PropertyChangeEvent;
import javax.swing.event.ChangeEvent;
import javax.accessibility.AccessibleRole;
import java.beans.PropertyChangeListener;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.UIResource;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.ComponentOrientation;
import java.beans.Transient;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.LayoutManager;
import javax.swing.plaf.ScrollPaneUI;
import java.awt.Point;
import java.awt.Component;
import javax.swing.border.Border;
import javax.accessibility.Accessible;

public class JScrollPane extends JComponent implements ScrollPaneConstants, Accessible
{
    private Border viewportBorder;
    private static final String uiClassID = "ScrollPaneUI";
    protected int verticalScrollBarPolicy;
    protected int horizontalScrollBarPolicy;
    protected JViewport viewport;
    protected JScrollBar verticalScrollBar;
    protected JScrollBar horizontalScrollBar;
    protected JViewport rowHeader;
    protected JViewport columnHeader;
    protected Component lowerLeft;
    protected Component lowerRight;
    protected Component upperLeft;
    protected Component upperRight;
    private boolean wheelScrollState;
    
    public JScrollPane(Component viewportView, int verticalScrollBarPolicy, int horizontalScrollBarPolicy) {
        this.verticalScrollBarPolicy = 20;
        this.horizontalScrollBarPolicy = 30;
        this.wheelScrollState = true;
        this.setLayout(new ScrollPaneLayout.UIResource());
        this.setVerticalScrollBarPolicy(verticalScrollBarPolicy);
        this.setHorizontalScrollBarPolicy(horizontalScrollBarPolicy);
        this.setViewport(this.createViewport());
        this.setVerticalScrollBar(this.createVerticalScrollBar());
        this.setHorizontalScrollBar(this.createHorizontalScrollBar());
        if (viewportView != null) {
            this.setViewportView(viewportView);
        }
        this.setUIProperty("opaque", Boolean.valueOf(true));
        this.updateUI();
        if (!this.getComponentOrientation().isLeftToRight()) {
            this.viewport.setViewPosition(new Point(Integer.MAX_VALUE, 0));
        }
    }
    
    public JScrollPane(Component component) {
        this(component, 20, 30);
    }
    
    public JScrollPane(int n, int n2) {
        this(null, n, n2);
    }
    
    public JScrollPane() {
        this(null, 20, 30);
    }
    
    public ScrollPaneUI getUI() {
        return (ScrollPaneUI)this.ui;
    }
    
    public void setUI(ScrollPaneUI ui) {
        super.setUI(ui);
    }
    
    @Override
    public void updateUI() {
        this.setUI((ScrollPaneUI)UIManager.getUI(this));
    }
    
    @Override
    public String getUIClassID() {
        return "ScrollPaneUI";
    }
    
    @Override
    public void setLayout(LayoutManager layoutManager) {
        if ((layoutManager instanceof ScrollPaneLayout)) {
            super.setLayout(layoutManager);
            ((ScrollPaneLayout)layoutManager).syncWithScrollPane(this);
        }
        else if (layoutManager == null) {
            super.setLayout(layoutManager);
        }
        else {
            throw new ClassCastException("layout of JScrollPane must be a ScrollPaneLayout");
        }
    }
    
    @Override
    public boolean isValidateRoot() {
        return true;
    }
    
    public int getVerticalScrollBarPolicy() {
        return this.verticalScrollBarPolicy;
    }
    
    public void setVerticalScrollBarPolicy(int verticalScrollBarPolicy) {
        switch (verticalScrollBarPolicy) {
            case 20:
            case 21:
            case 22: {
                this.firePropertyChange("verticalScrollBarPolicy", this.verticalScrollBarPolicy, this.verticalScrollBarPolicy = verticalScrollBarPolicy);
                this.revalidate();
                this.repaint();
                return;
            }
            default: {
                throw new IllegalArgumentException("invalid verticalScrollBarPolicy");
            }
        }
    }
    
    public int getHorizontalScrollBarPolicy() {
        return this.horizontalScrollBarPolicy;
    }
    
    public void setHorizontalScrollBarPolicy(int horizontalScrollBarPolicy) {
        switch (horizontalScrollBarPolicy) {
            case 30:
            case 31:
            case 32: {
                this.firePropertyChange("horizontalScrollBarPolicy", this.horizontalScrollBarPolicy, this.horizontalScrollBarPolicy = horizontalScrollBarPolicy);
                this.revalidate();
                this.repaint();
                return;
            }
            default: {
                throw new IllegalArgumentException("invalid horizontalScrollBarPolicy");
            }
        }
    }
    
    public Border getViewportBorder() {
        return this.viewportBorder;
    }
    
    public void setViewportBorder(Border viewportBorder) {
        this.firePropertyChange("viewportBorder", this.viewportBorder, this.viewportBorder = viewportBorder);
    }
    
    public Rectangle getViewportBorderBounds() {
        Rectangle rectangle = new Rectangle(this.getSize());
        Insets insets = this.getInsets();
        rectangle.x = insets.left;
        rectangle.y = insets.top;
        rectangle.width -= insets.left + insets.right;
        rectangle.height -= insets.top + insets.bottom;
        int leftToRight = SwingUtilities.isLeftToRight(this);
        JViewport columnHeader = this.getColumnHeader();
        if (columnHeader != null && columnHeader.isVisible()) {
            int height = columnHeader.getHeight();
            rectangle.y += height;
            rectangle.height -= height;
        }
        JViewport rowHeader = this.getRowHeader();
        if (rowHeader != null && rowHeader.isVisible()) {
            int width = rowHeader.getWidth();
            if (!(leftToRight == 0)) {
                rectangle.x += width;
            }
            rectangle.width -= width;
        }
        JScrollBar verticalScrollBar = this.getVerticalScrollBar();
        if (verticalScrollBar != null && verticalScrollBar.isVisible()) {
            int width2 = verticalScrollBar.getWidth();
            if (leftToRight == 0) {
                rectangle.x += width2;
            }
            rectangle.width -= width2;
        }
        JScrollBar horizontalScrollBar = this.getHorizontalScrollBar();
        if (horizontalScrollBar != null && horizontalScrollBar.isVisible()) {
            rectangle.height -= horizontalScrollBar.getHeight();
            return rectangle;
        }
        return rectangle;
    }
    
    public JScrollBar createHorizontalScrollBar() {
        return new JScrollPane.ScrollBar(0);
    }
    
    @Transient
    public JScrollBar getHorizontalScrollBar() {
        return this.horizontalScrollBar;
    }
    
    public void setHorizontalScrollBar(JScrollBar newValue) {
        JScrollBar horizontalScrollBar = this.getHorizontalScrollBar();
        this.horizontalScrollBar = newValue;
        if (newValue != null) {
            this.add(newValue, "HORIZONTAL_SCROLLBAR");
        }
        else if (horizontalScrollBar != null) {
            this.remove(horizontalScrollBar);
        }
        this.firePropertyChange("horizontalScrollBar", horizontalScrollBar, newValue);
        this.revalidate();
        this.repaint();
    }
    
    public JScrollBar createVerticalScrollBar() {
        return new JScrollPane.ScrollBar(1);
    }
    
    @Transient
    public JScrollBar getVerticalScrollBar() {
        return this.verticalScrollBar;
    }
    
    public void setVerticalScrollBar(JScrollBar scrollBar) {
        JScrollBar verticalScrollBar = this.getVerticalScrollBar();
        this.add(this.verticalScrollBar = scrollBar, "VERTICAL_SCROLLBAR");
        this.firePropertyChange("verticalScrollBar", verticalScrollBar, scrollBar);
        this.revalidate();
        this.repaint();
    }
    
    protected JViewport createViewport() {
        return new JViewport();
    }
    
    public JViewport getViewport() {
        return this.viewport;
    }
    
    public void setViewport(JViewport newValue) {
        JViewport viewport = this.getViewport();
        this.viewport = newValue;
        if (newValue != null) {
            this.add(newValue, "VIEWPORT");
        }
        else if (viewport != null) {
            this.remove(viewport);
        }
        this.firePropertyChange("viewport", viewport, newValue);
        if (this.accessibleContext != null) {
            ((JScrollPane.AccessibleJScrollPane)this.accessibleContext).resetViewPort();
        }
        this.revalidate();
        this.repaint();
    }
    
    public void setViewportView(Component view) {
        if (this.getViewport() == null) {
            this.setViewport(this.createViewport());
        }
        this.getViewport().setView(view);
    }
    
    @Transient
    public JViewport getRowHeader() {
        return this.rowHeader;
    }
    
    public void setRowHeader(JViewport newValue) {
        JViewport rowHeader = this.getRowHeader();
        this.rowHeader = newValue;
        if (newValue != null) {
            this.add(newValue, "ROW_HEADER");
        }
        else if (rowHeader != null) {
            this.remove(rowHeader);
        }
        this.firePropertyChange("rowHeader", rowHeader, newValue);
        this.revalidate();
        this.repaint();
    }
    
    public void setRowHeaderView(Component view) {
        if (this.getRowHeader() == null) {
            this.setRowHeader(this.createViewport());
        }
        this.getRowHeader().setView(view);
    }
    
    @Transient
    public JViewport getColumnHeader() {
        return this.columnHeader;
    }
    
    public void setColumnHeader(JViewport newValue) {
        JViewport columnHeader = this.getColumnHeader();
        this.columnHeader = newValue;
        if (newValue != null) {
            this.add(newValue, "COLUMN_HEADER");
        }
        else if (columnHeader != null) {
            this.remove(columnHeader);
        }
        this.firePropertyChange("columnHeader", columnHeader, newValue);
        this.revalidate();
        this.repaint();
    }
    
    public void setColumnHeaderView(Component view) {
        if (this.getColumnHeader() == null) {
            this.setColumnHeader(this.createViewport());
        }
        this.getColumnHeader().setView(view);
    }
    
    public Component getCorner(String s) {
        int leftToRight = this.getComponentOrientation().isLeftToRight();
        if (s.equals("LOWER_LEADING_CORNER")) {
            s = (leftToRight ? "LOWER_LEFT_CORNER" : "LOWER_RIGHT_CORNER");
        }
        else if (s.equals("LOWER_TRAILING_CORNER")) {
            s = (leftToRight ? "LOWER_RIGHT_CORNER" : "LOWER_LEFT_CORNER");
        }
        else if (s.equals("UPPER_LEADING_CORNER")) {
            s = (leftToRight ? "UPPER_LEFT_CORNER" : "UPPER_RIGHT_CORNER");
        }
        else if (s.equals("UPPER_TRAILING_CORNER")) {
            s = (leftToRight ? "UPPER_RIGHT_CORNER" : "UPPER_LEFT_CORNER");
        }
        if (s.equals("LOWER_LEFT_CORNER")) {
            return this.lowerLeft;
        }
        if (s.equals("LOWER_RIGHT_CORNER")) {
            return this.lowerRight;
        }
        if (s.equals("UPPER_LEFT_CORNER")) {
            return this.upperLeft;
        }
        if (s.equals("UPPER_RIGHT_CORNER")) {
            return this.upperRight;
        }
        return null;
    }
    
    public void setCorner(String s, Component component) {
        int leftToRight = this.getComponentOrientation().isLeftToRight();
        if (s.equals("LOWER_LEADING_CORNER")) {
            s = (leftToRight ? "LOWER_LEFT_CORNER" : "LOWER_RIGHT_CORNER");
        }
        else if (s.equals("LOWER_TRAILING_CORNER")) {
            s = (leftToRight ? "LOWER_RIGHT_CORNER" : "LOWER_LEFT_CORNER");
        }
        else if (s.equals("UPPER_LEADING_CORNER")) {
            s = (leftToRight ? "UPPER_LEFT_CORNER" : "UPPER_RIGHT_CORNER");
        }
        else if (s.equals("UPPER_TRAILING_CORNER")) {
            s = (leftToRight ? "UPPER_RIGHT_CORNER" : "UPPER_LEFT_CORNER");
        }
        Component component2;
        if (s.equals("LOWER_LEFT_CORNER")) {
            component2 = this.lowerLeft;
            this.lowerLeft = component;
        }
        else if (s.equals("LOWER_RIGHT_CORNER")) {
            component2 = this.lowerRight;
            this.lowerRight = component;
        }
        else if (s.equals("UPPER_LEFT_CORNER")) {
            component2 = this.upperLeft;
            this.upperLeft = component;
        }
        else if (s.equals("UPPER_RIGHT_CORNER")) {
            component2 = this.upperRight;
            this.upperRight = component;
        }
        else {
            throw new IllegalArgumentException("invalid corner key");
        }
        if (component2 != null) {
            this.remove(component2);
        }
        if (component != null) {
            this.add(component, s);
        }
        this.firePropertyChange(s, component2, component);
        this.revalidate();
        this.repaint();
    }
    
    @Override
    public void setComponentOrientation(ComponentOrientation componentOrientation) {
        super.setComponentOrientation(componentOrientation);
        if (this.verticalScrollBar != null) {
            this.verticalScrollBar.setComponentOrientation(componentOrientation);
        }
        if (this.horizontalScrollBar != null) {
            this.horizontalScrollBar.setComponentOrientation(componentOrientation);
        }
    }
    
    public boolean isWheelScrollingEnabled() {
        return this.wheelScrollState;
    }
    
    public void setWheelScrollingEnabled(boolean wheelScrollState) {
        this.firePropertyChange("wheelScrollingEnabled", this.wheelScrollState, this.wheelScrollState = wheelScrollState);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("ScrollPaneUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        String str = (this.viewportBorder != null) ? this.viewportBorder.toString() : "";
        String str2 = (this.viewport != null) ? this.viewport.toString() : "";
        String str3;
        if (this.verticalScrollBarPolicy == 20) {
            str3 = "VERTICAL_SCROLLBAR_AS_NEEDED";
        }
        else if (this.verticalScrollBarPolicy == 21) {
            str3 = "VERTICAL_SCROLLBAR_NEVER";
        }
        else if (this.verticalScrollBarPolicy == 22) {
            str3 = "VERTICAL_SCROLLBAR_ALWAYS";
        }
        else {
            str3 = "";
        }
        String str4;
        if (this.horizontalScrollBarPolicy == 30) {
            str4 = "HORIZONTAL_SCROLLBAR_AS_NEEDED";
        }
        else if (this.horizontalScrollBarPolicy == 31) {
            str4 = "HORIZONTAL_SCROLLBAR_NEVER";
        }
        else if (this.horizontalScrollBarPolicy == 32) {
            str4 = "HORIZONTAL_SCROLLBAR_ALWAYS";
        }
        else {
            str4 = "";
        }
        return super.paramString() + ",columnHeader=" + ((this.columnHeader != null) ? this.columnHeader.toString() : "") + ",horizontalScrollBar=" + ((this.horizontalScrollBar != null) ? this.horizontalScrollBar.toString() : "") + ",horizontalScrollBarPolicy=" + str4 + ",lowerLeft=" + ((this.lowerLeft != null) ? this.lowerLeft.toString() : "") + ",lowerRight=" + ((this.lowerRight != null) ? this.lowerRight.toString() : "") + ",rowHeader=" + ((this.rowHeader != null) ? this.rowHeader.toString() : "") + ",upperLeft=" + ((this.upperLeft != null) ? this.upperLeft.toString() : "") + ",upperRight=" + ((this.upperRight != null) ? this.upperRight.toString() : "") + ",verticalScrollBar=" + ((this.verticalScrollBar != null) ? this.verticalScrollBar.toString() : "") + ",verticalScrollBarPolicy=" + str3 + ",viewport=" + str2 + ",viewportBorder=" + str;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JScrollPane.AccessibleJScrollPane();
        }
        return this.accessibleContext;
    }
    
    protected class ScrollBar extends JScrollBar implements UIResource
    {
        private boolean unitIncrementSet;
        private boolean blockIncrementSet;
        
        public ScrollBar(int n) {
            super(n);
            this.putClientProperty("JScrollBar.fastWheelScrolling", Boolean.TRUE);
        }
        
        @Override
        public void setUnitIncrement(int unitIncrement) {
            this.unitIncrementSet = true;
            this.putClientProperty("JScrollBar.fastWheelScrolling", null);
            super.setUnitIncrement(unitIncrement);
        }
        
        @Override
        public int getUnitIncrement(int n) {
            JViewport viewport = JScrollPane.this.getViewport();
            if (!this.unitIncrementSet && viewport != null && viewport.getView() instanceof Scrollable) {
                return ((Scrollable)(Scrollable)viewport.getView()).getScrollableUnitIncrement(viewport.getViewRect(), this.getOrientation(), n);
            }
            return super.getUnitIncrement(n);
        }
        
        @Override
        public void setBlockIncrement(int blockIncrement) {
            this.blockIncrementSet = true;
            this.putClientProperty("JScrollBar.fastWheelScrolling", null);
            super.setBlockIncrement(blockIncrement);
        }
        
        @Override
        public int getBlockIncrement(int n) {
            JViewport viewport = JScrollPane.this.getViewport();
            if (this.blockIncrementSet || viewport == null) {
                return super.getBlockIncrement(n);
            }
            if ((viewport.getView() instanceof Scrollable)) {
                return ((Scrollable)(Scrollable)viewport.getView()).getScrollableBlockIncrement(viewport.getViewRect(), this.getOrientation(), n);
            }
            if (this.getOrientation() == 1) {
                return viewport.getExtentSize().height;
            }
            return viewport.getExtentSize().width;
        }
    }
    
    protected class AccessibleJScrollPane extends JComponent.AccessibleJComponent implements ChangeListener, PropertyChangeListener
    {
        protected JViewport viewPort;
        
        public void resetViewPort() {
            if (this.viewPort != null) {
                this.viewPort.removeChangeListener(this);
                this.viewPort.removePropertyChangeListener(this);
            }
            this.viewPort = JScrollPane.this.getViewport();
            if (this.viewPort != null) {
                this.viewPort.addChangeListener(this);
                this.viewPort.addPropertyChangeListener(this);
            }
        }
        
        public AccessibleJScrollPane() {
            this.viewPort = null;
            this.resetViewPort();
            JScrollBar horizontalScrollBar = JScrollPane.this.getHorizontalScrollBar();
            if (horizontalScrollBar != null) {
                this.setScrollBarRelations(horizontalScrollBar);
            }
            JScrollBar verticalScrollBar = JScrollPane.this.getVerticalScrollBar();
            if (verticalScrollBar != null) {
                this.setScrollBarRelations(verticalScrollBar);
            }
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.SCROLL_PANE;
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            if (changeEvent == null) {
                throw new NullPointerException();
            }
            this.firePropertyChange("AccessibleVisibleData", Boolean.valueOf(false), Boolean.valueOf(true));
            return;
        }
        
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if ((propertyName == "horizontalScrollBar" || propertyName == "verticalScrollBar") && propertyChangeEvent.getNewValue() instanceof JScrollBar) {
                this.setScrollBarRelations((JScrollBar)propertyChangeEvent.getNewValue());
            }
        }
        
        void setScrollBarRelations(JScrollBar target) {
            AccessibleRelation relation = new AccessibleRelation(AccessibleRelation.CONTROLLED_BY, target);
            target.getAccessibleContext().getAccessibleRelationSet().add(new AccessibleRelation(AccessibleRelation.CONTROLLER_FOR, JScrollPane.this));
            this.getAccessibleRelationSet().add(relation);
        }
    }
}
